#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <errno.h>
#include <mqueue.h>
#include <gmp.h>
#include <semaphore.h>
#include <sys/mman.h>
#include "Enc.h"

int main(int argc, char *argv[])
{
  int c;
  char *eval = NULL;

  while(( c = getopt(argc, argv, "rme:k"))!=-1)
    switch(c){
      case 'r':
        printf("Invoke the sender, router, destination\n");
        start_nodes();
        break;
      case 'm':
        parent_to_sender_msg();
        break;
      case 'e':
        eval = optarg;

        if (strcmp(eval, "df96")  == 0){
          set_encryption_scheme(DF96);
          set_df_add_mul_ph_keys();
        } else if (strcmp(eval, "df98") == 0) {
          set_encryption_scheme(DF98);
          set_df_new_ph_keys();
        } else if (strcmp(eval, "df02") == 0) {
          set_encryption_scheme(DF02);
          set_df_field_mmh_ph_keys();
        } else if(strcmp(eval, "mmh") ==0){
          set_encryption_scheme(MMH);
          set_df_field_mmh_ph_keys();
        }else{
          fprintf (stderr, "Invalid arguments to option e Valid arguments: df96, df98, df02, mmh \n");
        }
        break;
      case '?':
        fprintf (stderr, "Unknown option character `\\x%x'.\n", optopt);
        return 1;
      default:
        abort();
    }
  return 0;
}

void set_encryption_scheme(enc_scheme encryption_scheme){

  /* Reading the shared Memory*/
  symmetric_key *sk_ptr;
  sk_ptr = get_symmetric_key();

  enc_mutex = sem_open(ENC_KEYS_TABLE_LOCK, 0);
  sem_wait(enc_mutex);
    sk_ptr ->Ph = encryption_scheme;
  sem_post(enc_mutex);
  sem_close(enc_mutex);
}

enc_scheme get_encryption_scheme(){
  /* Reading the shared Memory*/
  enc_scheme encryption_scheme;
  symmetric_key *sk_ptr;
  sk_ptr = get_symmetric_key();

  enc_mutex = sem_open(ENC_KEYS_TABLE_LOCK, 0);
  sem_wait(enc_mutex);
   encryption_scheme = sk_ptr->Ph;
  sem_post(enc_mutex);
  sem_close(enc_mutex);
  munmap(sk_ptr, sizeof(symmetric_key));
  return encryption_scheme;

}


void set_df_add_mul_ph_keys(){
  int cmp, comp;
  int d;
  mpz_t md, m, r, r_inverse;
  long seed;
  gmp_randstate_t state;

  mpz_init(md);
  mpz_init(m);
  mpz_init(r);
  mpz_init(r_inverse);


  printf("wsns_simulation> Enter the r value : ");
  mpz_inp_str(r, NULL, 10);

  printf("wsns_simulation> Enter m' value : ");
  mpz_inp_str(md, NULL, 10);

  printf("wsns_simulation> Enter the m value : ");
  mpz_inp_str(m, NULL, 10);

  printf("wsns_simulation> Enter d value : ");
  scanf("%d", &d);
  
  gmp_randinit_default(state);

  seed = get_seed();
  gmp_randseed_ui(state, seed);

  /* Opening the shared Memory and setting the key values in Shared Memory */
  symmetric_key *sk_ptr;
  sk_ptr = get_symmetric_key();

  enc_mutex = sem_open(ENC_KEYS_TABLE_LOCK, 0);
  sem_wait(enc_mutex);
    long size = sizeof(char);
    long numb = 8*size - 0;

    sk_ptr->r_count = (mpz_sizeinbase (r, 2) + numb-1) / numb;
    sk_ptr->md_count = (mpz_sizeinbase (md, 2) + numb-1) / numb;
    sk_ptr->m_count = (mpz_sizeinbase (m, 2) + numb-1) / numb;

    mpz_export(sk_ptr->r_buff, NULL, 1, 1, 0, 0, r);
    mpz_export(sk_ptr->md_buff, NULL, 1, 1, 0, 0, md);
    mpz_export(sk_ptr->m_buff, NULL, 1, 1, 0, 0, m);
    sk_ptr->d =d;
  sem_post(enc_mutex);
  sem_close(enc_mutex);

  mpz_clear(md);
  mpz_clear(m);
  mpz_clear(r);
  mpz_clear(r_inverse);
  munmap(sk_ptr, sizeof(symmetric_key));

}

void set_df_new_ph_keys(){
  int d;
  mpz_t p, q, m, rp, rq, p_prime, q_prime;

  mpz_init(p);
  mpz_init(rp);
  mpz_init(p_prime);
  mpz_init(q);
  mpz_init(rq);
  mpz_init(q_prime);
  mpz_init(m);

  printf("wsns_simulation> Enter p value : ");
  mpz_inp_str(p_prime, NULL, 10);

  printf("wsns_simulation> Enter rp value : ");
  mpz_inp_str(rp, NULL, 10);

  printf("wsns_simulation> Enter q value : ");
  mpz_inp_str(q_prime, NULL, 10);

  printf("wsns_simulation> Enter rq value : ");
  mpz_inp_str(rq, NULL, 10);

  printf("wsns_simulation> Enter d value : ");
  scanf("%d", &d);

  mpz_mul(m, p_prime, q_prime);

  /* Opening the shared Memory and setting the key values in Shared Memory */
  symmetric_key *sk_ptr;
  sk_ptr = get_symmetric_key();

  enc_mutex = sem_open(ENC_KEYS_TABLE_LOCK, 0);
  sem_wait(enc_mutex);
    long size = sizeof(char);
    long numb = 8*size - 0;

    sk_ptr->p_count = (mpz_sizeinbase (p_prime, 2) + numb-1) / numb;
    sk_ptr->q_count = (mpz_sizeinbase (q_prime, 2) + numb-1) / numb;
    sk_ptr->rp_count = (mpz_sizeinbase (rp, 2) + numb-1) / numb;
    sk_ptr->rq_count = (mpz_sizeinbase (rq, 2) + numb-1) / numb;
    sk_ptr->mpq_count = (mpz_sizeinbase (m, 2) + numb-1) / numb;

    mpz_export(sk_ptr->p_buff, NULL, 1, 1, 0, 0, p_prime);
    mpz_export(sk_ptr->q_buff, NULL, 1, 1, 0, 0, q_prime);
    mpz_export(sk_ptr->rp_buff, NULL, 1, 1, 0, 0, rp);
    mpz_export(sk_ptr->rq_buff, NULL, 1, 1, 0, 0, rq);
    mpz_export(sk_ptr->mpq_buff, NULL, 1, 1, 0, 0, m);
    sk_ptr->d=d;
  sem_post(enc_mutex);
  sem_close(enc_mutex);

  mpz_clear(p);
  mpz_clear(rp);
  mpz_clear(p_prime);
  mpz_clear(q);
  mpz_clear(rq);
  mpz_clear(q_prime);
  mpz_clear(m);
  munmap(sk_ptr, sizeof(symmetric_key));
}

void set_df_field_mmh_ph_keys(){
  mpz_t p, q, m, p_prime, q_prime, r;

  mpz_init(p);
  mpz_init(p_prime);
  mpz_init(q);
  mpz_init(q_prime);
  mpz_init(m);
  mpz_init(r);

  printf("wsns_simulation> Enter p value : ");
  mpz_inp_str(p_prime, NULL, 10);

  printf("wsns_simulation> Enter q value : ");
  mpz_inp_str(q_prime, NULL, 10);

  mpz_mul(m, p_prime, q_prime);


  /* Opening the shared Memory and setting the key values in Shared Memory */
  symmetric_key *sk_ptr;
  sk_ptr = get_symmetric_key();

  enc_mutex = sem_open(ENC_KEYS_TABLE_LOCK, 0);
  sem_wait(enc_mutex);

    long size = sizeof(char);
    long numb = 8*size - 0;

    sk_ptr->p_count = (mpz_sizeinbase (p_prime, 2) + numb-1) / numb;
    sk_ptr->q_count = (mpz_sizeinbase (q_prime, 2) + numb-1) / numb;
    sk_ptr->mpq_count = (mpz_sizeinbase (m, 2) + numb-1) / numb;

    mpz_export(sk_ptr->p_buff, NULL, 1, 1, 0, 0, p_prime);
    mpz_export(sk_ptr->q_buff, NULL, 1, 1, 0, 0, q_prime);
    mpz_export(sk_ptr->mpq_buff, NULL, 1, 1, 0, 0, m);

  sem_post(enc_mutex);
  sem_close(enc_mutex);

  mpz_clear(p);
  mpz_clear(p_prime);
  mpz_clear(q);
  mpz_clear(q_prime);
  mpz_clear(m);
  mpz_clear(r);
  munmap(sk_ptr, sizeof(symmetric_key));
}

void get_various_encryption_keys(mpz_symmetric_key *mpz_keys_ptr){

  symmetric_key *sk_ptr;

  mpz_init(mpz_keys_ptr->r);
  mpz_init(mpz_keys_ptr->md);
  mpz_init(mpz_keys_ptr->m);
  mpz_init(mpz_keys_ptr->p);
  mpz_init(mpz_keys_ptr->q);
  mpz_init(mpz_keys_ptr->rp);
  mpz_init(mpz_keys_ptr->rq);
  mpz_init(mpz_keys_ptr->mpq);

  /* Reading from Shared memory */
  sk_ptr = get_symmetric_key();

  enc_mutex = sem_open(ENC_KEYS_TABLE_LOCK, 0);
  sem_wait(enc_mutex);

    printf(" importing The r key \n");
    mpz_import(mpz_keys_ptr->r, sk_ptr->r_count, 1, 1, 0, 0, sk_ptr->r_buff);
    gmp_printf(" The r key :%Zd", mpz_keys_ptr->r);
    mpz_import(mpz_keys_ptr->md, sk_ptr->md_count, 1, 1, 0, 0, sk_ptr->md_buff);
    mpz_import(mpz_keys_ptr->m, sk_ptr->m_count, 1, 1, 0, 0, sk_ptr->m_buff);

    mpz_import(mpz_keys_ptr->p, sk_ptr->p_count, 1, 1, 0, 0, sk_ptr->p_buff);
    mpz_import(mpz_keys_ptr->rp, sk_ptr->rp_count, 1, 1, 0, 0, sk_ptr->rp_buff);
    mpz_import(mpz_keys_ptr->q, sk_ptr->q_count, 1, 1, 0, 0, sk_ptr->q_buff);
    mpz_import(mpz_keys_ptr->rq, sk_ptr->rq_count, 1, 1, 0, 0, sk_ptr->rq_buff);
    mpz_import(mpz_keys_ptr->mpq, sk_ptr->mpq_count, 1, 1, 0, 0, 
               sk_ptr->mpq_buff);
    mpz_keys_ptr->d = sk_ptr->d;
    mpz_keys_ptr->Ph = sk_ptr->Ph;

  sem_post(enc_mutex);
  sem_close(enc_mutex);
  munmap(sk_ptr, sizeof(symmetric_key));

}

void set_df_mul_ph_rd_keys(mpz_t value){
  int cmp=0;
  int d;
  mpz_t r,md,m;
  mpz_init(r);
  mpz_init(md);
  mpz_init(m);

  printf("wsns_simulation> ");
  if( mpz_cmp_ui(value, 11) == 0){
    printf(" Enter the r value : ");
    mpz_inp_str(r, NULL, 10);
  }

  if( mpz_cmp_ui(value, 12) == 0){
    printf(" Enter the m' value : ");
    mpz_inp_str(md, NULL, 10);
  }

  if( mpz_cmp_ui(value, 13) == 0){
    printf(" Enter the m value : ");
    mpz_inp_str(m, NULL, 10);
  }
  
  if( mpz_cmp_ui(value, 14) == 0){
      printf(" Enter the d value : ");
      scanf("%d", &d);
  }



  /* Opening the shared Memory and setting the r or d key in Shared Memory */
  symmetric_key *sk_ptr;
  sk_ptr = get_symmetric_key();

  enc_mutex = sem_open(ENC_KEYS_TABLE_LOCK, 0);
  sem_wait(enc_mutex);
    long size = sizeof(char);
    long numb = 8*size - 0;

    if(mpz_cmp_ui(value, 11) == 0){ 
      sk_ptr->r_count = (mpz_sizeinbase (r, 2) + numb-1) / numb;
      mpz_export(sk_ptr->r_buff, NULL, 1, 1, 0, 0, r);
    }  
    if(mpz_cmp_ui(value, 12) == 0){ 
      sk_ptr->md_count = (mpz_sizeinbase (md, 2) + numb-1) / numb;
      mpz_export(sk_ptr->md_buff, NULL, 1, 1, 0, 0, md);
    }  

    if(mpz_cmp_ui(value, 13) == 0){ 
      sk_ptr->m_count = (mpz_sizeinbase (m, 2) + numb-1) / numb;
      mpz_export(sk_ptr->m_buff, NULL, 1, 1, 0, 0, m);
    }  

    if(mpz_cmp_ui(value, 14) == 0){ 
      sk_ptr->d = d;
    }

  sem_post(enc_mutex);
  sem_close(enc_mutex);

  mpz_clear(r);
  munmap(sk_ptr, sizeof(symmetric_key));

}

void set_df_new_ph_rp_p_d_keys(mpz_t value){
  int cmp=0;
  int d;
  mpz_t rp, rq, p, mpq, q;

  mpz_init(rp);
  mpz_init(rq);
  mpz_init(p);
  mpz_init(q);
  mpz_init(mpq);

  printf("wsns_simulation> ");

  if(mpz_cmp_ui(value, 11)==0){ 
    printf(" Enter the p value : ");
    mpz_inp_str(p, NULL, 10);
    gmp_printf(" The p value: %Zd\n", p);
  }

  if(mpz_cmp_ui(value, 12)==0){ 
    printf(" Enter the rp value : ");
    mpz_inp_str(rp, NULL, 10);
    gmp_printf(" The rp value: %Zd\n", rp);
  }

  if(mpz_cmp_ui(value, 13)==0){ 
    printf(" Enter the q value : ");
    mpz_inp_str(q, NULL, 10);
    gmp_printf(" The q value: %Zd\n", q);
  }

  if(mpz_cmp_ui(value, 14)==0){ 
    printf(" Enter the rq value : ");
    mpz_inp_str(rq, NULL, 10);
    gmp_printf(" The rq value: %Zd\n", rq);
  }

  if(mpz_cmp_ui(value, 15)==0){ 
    printf(" Enter the d value : ");
    scanf("%d", &d);
  }


  /* Opening the shared Memory and setting the r or d key in Shared Memory */
  symmetric_key *sk_ptr;
  sk_ptr = get_symmetric_key();

  enc_mutex = sem_open(ENC_KEYS_TABLE_LOCK, 0);
  sem_wait(enc_mutex);
    long size = sizeof(char);
    long numb = 8*size - 0;

    if(mpz_cmp_ui(value, 11)==0){ 
      sk_ptr->p_count = (mpz_sizeinbase (p, 2) + numb-1) / numb;
      mpz_export(sk_ptr->p_buff, NULL, 1, 1, 0, 0, p);
      mpz_import(q, sk_ptr->q_count, 1, 1, 0, 0, sk_ptr->q_buff);
      mpz_mul(mpq, p, q);
      sk_ptr->mpq_count = (mpz_sizeinbase (mpq, 2) + numb-1) / numb;
      mpz_export(sk_ptr->mpq_buff, NULL, 1, 1, 0, 0, mpq);
    }  

    if(mpz_cmp_ui(value, 12)==0){ 
      sk_ptr->rp_count = (mpz_sizeinbase (rp, 2) + numb-1) / numb;
      mpz_export(sk_ptr->rp_buff, NULL, 1, 1, 0, 0, rp);
    }  

    if(mpz_cmp_ui(value, 13)==0){ 
      sk_ptr->q_count = (mpz_sizeinbase (q, 2) + numb-1) / numb;
      mpz_export(sk_ptr->q_buff, NULL, 1, 1, 0, 0, q);
      mpz_import(p, sk_ptr->p_count, 1, 1, 0, 0, sk_ptr->p_buff);
      mpz_mul(mpq, p, q);
      sk_ptr->mpq_count = (mpz_sizeinbase (mpq, 2) + numb-1) / numb;
      mpz_export(sk_ptr->mpq_buff, NULL, 1, 1, 0, 0, mpq);
    }  

    if(mpz_cmp_ui(value, 14)==0){ 
      sk_ptr->rq_count = (mpz_sizeinbase (rq, 2) + numb-1) / numb;
      mpz_export(sk_ptr->rq_buff, NULL, 1, 1, 0, 0, rq);
    }  

    if(mpz_cmp_ui(value, 15)==0){ 
      sk_ptr->d = d;
    }

  sem_post(enc_mutex);
  sem_close(enc_mutex);

  mpz_clear(rp);
  mpz_clear(p);
  mpz_clear(q);
  mpz_clear(mpq);
  munmap(sk_ptr, sizeof(symmetric_key));

}

void set_df_field_mmh_ph_p_r_keys(mpz_t value){
  int cmp=0;
  mpz_t p, mpq, q, r;

  mpz_init(p);
  mpz_init(q);
  mpz_init(r);
  mpz_init(mpq);

  printf("wsns_simulation> ");

  if(mpz_cmp_ui(value, 11)==0){ 
    printf(" Enter the p value : ");
    mpz_inp_str(p, NULL, 10);
    gmp_printf(" The p value: %Zd\n", p);
  }


  /* Opening the shared Memory and setting the r or d key in Shared Memory */
  symmetric_key *sk_ptr;
  sk_ptr = get_symmetric_key();

  enc_mutex = sem_open(ENC_KEYS_TABLE_LOCK, 0);
  sem_wait(enc_mutex);
    long size = sizeof(char);
    long numb = 8*size - 0;

    if(mpz_cmp_ui(value, 11)==0){ 
      sk_ptr->p_count = (mpz_sizeinbase (p, 2) + numb-1) / numb;
      mpz_export(sk_ptr->p_buff, NULL, 1, 1, 0, 0, p);
      mpz_import(q, sk_ptr->q_count, 1, 1, 0, 0, sk_ptr->q_buff);
      mpz_mul(mpq, p, q);
      sk_ptr->mpq_count = (mpz_sizeinbase (mpq, 2) + numb-1) / numb;
      mpz_export(sk_ptr->mpq_buff, NULL, 1, 1, 0, 0, mpq);
    }  

  sem_post(enc_mutex);
  sem_close(enc_mutex);

  mpz_clear(p);
  mpz_clear(q);
  mpz_clear(r);
  mpz_clear(mpq);
  munmap(sk_ptr, sizeof(symmetric_key));

}

symmetric_key *get_symmetric_key(){
  /* Opening the shared Memory  */

  int sk_d;
  symmetric_key *sk_ptr;

  sk_d = shm_open(ENC_KEYS_TABLE, O_RDWR, 0666);
  sk_ptr = mmap(NULL, sizeof(symmetric_key), PROT_READ|PROT_WRITE, 
                  MAP_SHARED, sk_d, 0);
  close(sk_d);
  return sk_ptr;
}


void decrypt(mpz_t encryption_msg_split_array[][255], mpz_t plaintext, 
             int previous_counter, mpz_symmetric_key *mpz_keys_ptr)
{

   FILE *decrypt_time_file_handle;
   decrypt_time_file_handle = fopen(manets_log, "a+");


  if(mpz_keys_ptr->Ph == DF96){
    printf(" The Encryption is PH addive Multiplicative\n");
    df_add_mul_Ph_decryption(encryption_msg_split_array, plaintext, 
                             previous_counter,
                             mpz_keys_ptr, decrypt_time_file_handle);
  }  
  if(mpz_keys_ptr->Ph == DF98){
    printf(" The Encryption is New PH\n");
    df_new_Ph_decryption(encryption_msg_split_array, plaintext, 
                         previous_counter, 
                         mpz_keys_ptr, decrypt_time_file_handle);
  }  
  if(mpz_keys_ptr->Ph == MMH){
    printf(" The Encryption is MMH\n");
    mmh_decryption(encryption_msg_split_array, plaintext, previous_counter, 
                   mpz_keys_ptr, decrypt_time_file_handle);
  }  
  if(mpz_keys_ptr->Ph == DF02){
    printf(" Entering the field decryption\n");
    printf(" The Encryption is PH Mixed Multiplicative\n");
    df_field_decryption(encryption_msg_split_array, plaintext, previous_counter, 
                        mpz_keys_ptr, decrypt_time_file_handle);
    printf(" Done with field decryption\n");
  }  

  fclose(decrypt_time_file_handle);

}

