//***************************************************************************************
//*  Copyright: National ICT Australia,  2007, 2008					*
//*  Developed at the Networks and Pervasive Computing program				*
//*  Author(s): Athanassios Boulis, Dimosthenis Pediaditakis				*
//*  This file is distributed under the terms in the attached LICENSE file.		*
//*  If you do not find this file, copies can be found by writing to:			*
//*											*
//*      NICTA, Locked Bag 9013, Alexandria, NSW 1435, Australia			*
//*      Attention:  License Inquiry.							*
//*											*
//***************************************************************************************





#ifndef _WCHANNEL_INTERFERENCE_H
#define _WCHANNEL_INTERFERENCE_H


#include "WChannelGenericMessage_m.h"
#include "WCsupportFunctions.h"
#include "MacGenericFrame_m.h"
#include "DebugInfoWriter.h"
#include <omnetpp.h>
using namespace std;



enum CollisionModel
{
	NO_INTERFERENCE_NO_COLLISIONS = 8101,
	SIMPLE_COLLISION_MODEL = 8102,
	ADDITIVE_INTERFERENCE_MODEL = 8103,
	COMPLEX_INTERFERENCE_MODEL = 8104
};


enum modulationTypeAvail
{
	MODULATION_FSK = 8104,
	MODULATION_PSK = 8105,
	MODULATION_IDEAL = 8106
};


enum encodingTypeAvail
{
	ENCODING_NRZ = 8107,
	ENCODING_4B5B = 8108,
	ENCODING_MANCHESTER = 8109,
	ENCODING_SECDEC = 8110
};


class WChannel_Interference : public cSimpleModule {
	private:
		// parameters and variables
		
		/*--- The .ned file's parameters ---*/
		bool printStatistics;
		bool printDebugInfo;
		int numNodes;				// the number of nodes
		
		// Wireless Channel module parameters
		double pathLossExponent;	// the path loss exponent
		double noiseFloor;			// in dBm
		double PLd0;				// Power loss at a reference distance d0 (in dBm)
		double d0;					// reference distance (in meters)
		int collisionModel;
		bool allBidirectionalLinks;
		double sigma;				// std of a zero-mean Gaussian RV
			
		// Radio module parameters		
		double dataRate;
		double noiseBandwidth;
		int modulationType;
		int encodingType;
		double receiverSensitivity;
		int numTxPowerLevels;		// the number of different Tx Power levels (max is 15)
		double txPowerLevels[15];	// the array holding the different Tx PowerLevels,
									// txPowerLevel[0] is the default
		
		/*--- Custom class member variables ---*/
		
		double thresholdSNRdB;		// below this SNR (in dB) we assume reception in not possible
		
		double goodLinkSNRdB;		// above this SNR (in dB) we assume reception is very possible
				
		double ***rxSignal;			// N by N by L array stores the received signal power (in dB)
									// N = numNodes, L = numTxPowerLevels
									// rxSignal[i][j][k] is the received signal power at node j
		                   			// when node i is transmitting with TxPowerLevel k
		                   			
		double **maxInterferenceForCurrentTx;   // maxInterferenceForCurrentTx[i][j] is the interference
												// power (in dBm) witnessed by node j for the *current*
												// transmission of node i. Array N by N.
												// The array needed to be only A by N, where A is the number
												// of active (transmitting) nodes but N by N helps with
												// fast indexing
												
		ListElement *activeNodes;				// a linked-list containing active (transmitting) nodes
		
		ListElement *carrierSensingNodes;		// a linked-list containing nodes who are carrier sensing
		
		int **stats;							// N by 5, holds various statistics:
												// stats[i][0] is transmissions from node i
												// stats[i][1] is the possible receptions for the transmissions of node i
												// stats[i][2] is the interferred receptions for the transmissions of node i
												// stats[i][3] is the actual receptions for the transmissions of node i
												// stats[i][4] is the receptions of node i
		
		const char * rxSignal_ConnectivityMap;
		const char * PRR_ConnectivityMap;
		int defaultPowerLevel;
		double minumumPacketSize;				//in bytes
		

	protected:
		virtual void initialize();
		virtual void handleMessage(cMessage *msg);
		virtual void finish();
		
		void readIniFileParameters(void);
		void parseRxPowerMap(void);
		void parsePrrMap(void);
		void printRxSignalTable(void);
};


#endif //_WCHANNEL_INTERFERENCE_H
