//***************************************************************************************
//*  Copyright: National ICT Australia,  2007, 2008					*
//*  Developed at the Networks and Pervasive Computing program				*
//*  Author(s): Athanassios Boulis, Dimosthenis Pediaditakis				*
//*  This file is distributed under the terms in the attached LICENSE file.		*
//*  If you do not find this file, copies can be found by writing to:			*
//*											*
//*      NICTA, Locked Bag 9013, Alexandria, NSW 1435, Australia			*
//*      Attention:  License Inquiry.							*
//*											*
//***************************************************************************************



#include "WCsupportFunctions.h"

#define ERFINV_ERROR 100000.0


ListElement * addToFront(ListElement *n, ListElement *newListElement)
{
	newListElement->next = n;
	return newListElement;
}


ListElement * deleteListElement(ListElement* start, int ind)
{ //the start index of the linked-list is start
	ListElement *n, *p;

	if (start == NULL) {
		return NULL;
	}

	if (start -> index == ind){
		p = start->next;
		delete start;
		return p;
	}

	for (n = start; n->next != NULL; n = n->next) {
		p = n->next;
		if (p->index == ind) {
			n->next = p->next;
			delete p;
			return start;
		}
	}
	return start;
}


// approximates the addition of 2 signals expressed in dBm. Value returned in dBm
double addPower_dBm(double a, double b)
{
	double diff = a - b;
	// accurate additions for specific diff values
	// A = 10^(a/10), B = 10^(b/10), A/B = 10^(diff/10)
	// M = max(A,B), m = min(A,B)
	// sum = M*(1+ m/M)),  dB_add_to_max = 10*log(1+ m/M)
	// For diff -> dB_add_to_max
	// 0 -> 3
	// 1 -> 2.52
	// 2 -> 2.12
	// 3 -> 1.76
	// 4 -> 1.46
	// 5 -> 1.17
	// 6 -> 1
	// 7 -> 0.79

	if (diff > 7.0)  return a;
	if (diff < -7.0) return b;
	if (diff > 5.0)  return (a + 1.0);
	if (diff < -5.0) return (b + 1.0);
	if (diff > 3.0)  return (a + 1.5);
	if (diff < -3.0) return (b + 1.5);
	if (diff > 2.0)  return (a + 2.0);
	if (diff < -2.0) return (b + 2.0);
	if (diff > 1.0)  return (a + 2.5);
	if (diff < -1.0) return (b + 2.5);
	if (diff > 0.0)  return (a + 3.0);
	return (b + 3.0);
}


double erfInv( double y )
{
	static double a[] = {0.0,  0.886226899, -1.645349621,  0.914624893, -0.140543331 };
	static double b[] = {0.0, -2.118377725,  1.442710462, -0.329097515,  0.012229801 };
	static double c[] = {0.0, -1.970840454, -1.624906493,  3.429567803,  1.641345311 };
	static double d[] = {0.0,  3.543889200,  1.637067800 };
	
	double x, z;
  
 	if ( y > -1. ) 
  	{
  		if ( y >= -.7 ) 
  		{
  			if ( y <= .7 ) 
  			{
				z = y*y;
				x = y * (((a[4]*z+a[3])*z+a[2])*z+a[1]) / ((((b[4]*z+b[3])*z+b[2])*z+b[1])*z+1.0);
	  		}
	  		else
	  		if ( y < 1 ) 
	  		{
				z = sqrt(-log((1-y)/2.0));
				x = (((c[4]*z+c[3])*z+c[2])*z+c[1]) / ((d[2]*z+d[1])*z+1.0);
	  		}
	  		else 
	  			return ERFINV_ERROR;
		}
		else
		{
			z = sqrt(-log((1+y)/2.0));
	  		x = -(((c[4]*z+c[3])*z+c[2])*z+c[1]) / ((d[2]*z+d[1])*z+1);
		}
	}
	else 
		return ERFINV_ERROR;
  	
  	return x;
}


double erfcInv( double y )
{
	double result;
	result  = erfInv(1.0-y);
  	return result;
}
